// This addon causes the default browser check on first run to delay by a
// configured about (e.g. 60 minutes).  In certain distrutions it is not
// desirable to immediatly ask the user to choose the default, rather allow
// them to explore a bit first.  Subsequent (after first) runs will revert to
// the default Firefox behaviour.
//
// This addon is used in specific partner builds where two default preferences,
// extensions.partnerdefaults.checkdefaults and extensions.partnerdefaults.firstRun,
// would be added via the distribution.ini file for the repack.

const {classes: Cc, interfaces: Ci, utils: Cu} = Components;
const MILLISECONDS_PER_DAY = 24 * 60 * 60 * 1000;
const OLDEST_ALLOWED_YEAR = 2015;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://services-common/utils.js");
Cu.import("resource://gre/modules/Preferences.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "Services",
                                  "resource://gre/modules/Services.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "RecentWindow",
                                  "resource:///modules/RecentWindow.jsm");
XPCOMUtils.defineLazyGetter(this, "ShellService", function() {
  try {
    return Cc["@mozilla.org/browser/shell-service;1"].
           getService(Ci.nsIShellService);
  }
  catch(ex) {
    return null;
  }
});

// unfortunately, DefaultBrowserCheck in nsBrowserGlue.js is not exported in a
// resusable way.  Copied here for the interim.
// DefaultBrowserCheck.setAsDefault is slightly changed to handle the windows
// OS/version check at run time, the original in nsBrowserGlue.js has a
// preprocessor check.
var DefaultBrowserCheck = {
  get OPTIONPOPUP() { return "defaultBrowserNotificationPopup" },

  closePrompt: function(aNode) {
    if (this._notification) {
      this._notification.close();
    }
  },

  setAsDefault: function() {
    let claimAllTypes = true;
    try {
      // In Windows 8+, the UI for selecting default protocol is much
      // nicer than the UI for setting file type associations. So we
      // only show the protocol association screen on Windows 8+.
      // Windows 8 is version 6.2.
      if (Services.sysinfo.getPropertyAsAString("name") == "Windows_NT") {
        let version = Services.sysinfo.getProperty("version");
        claimAllTypes = (parseFloat(version) < 6.2);
      }
    } catch (ex) { }
    try {
      ShellService.setDefaultBrowser(claimAllTypes, false);
    } catch (ex) {
      Cu.reportError(ex);
    }
  },

  _createPopup: function(win, notNowStrings, neverStrings) {
    let doc = win.document;
    let popup = doc.createElement("menupopup");
    popup.id = this.OPTIONPOPUP;

    let notNowItem = doc.createElement("menuitem");
    notNowItem.id = "defaultBrowserNotNow";
    notNowItem.setAttribute("label", notNowStrings.label);
    notNowItem.setAttribute("accesskey", notNowStrings.accesskey);
    popup.appendChild(notNowItem);

    let neverItem = doc.createElement("menuitem");
    neverItem.id = "defaultBrowserNever";
    neverItem.setAttribute("label", neverStrings.label);
    neverItem.setAttribute("accesskey", neverStrings.accesskey);
    popup.appendChild(neverItem);

    popup.addEventListener("command", this);

    let popupset = doc.getElementById("mainPopupSet");
    popupset.appendChild(popup);
  },

  handleEvent: function(event) {
    if (event.type == "command") {
      if (event.target.id == "defaultBrowserNever") {
        if (ShellService) {
          ShellService.shouldCheckDefaultBrowser = false;
        }
      }
      this.closePrompt();
    }
  },

  prompt: function(win) {
    let useNotificationBar = Services.prefs.getBoolPref("browser.defaultbrowser.notificationbar");

    let brandBundle = win.document.getElementById("bundle_brand");
    let brandShortName = brandBundle.getString("brandShortName");

    let shellBundle = win.document.getElementById("bundle_shell");
    let buttonPrefix = "setDefaultBrowser" + (useNotificationBar ? "" : "Alert");
    let yesButton = shellBundle.getFormattedString(buttonPrefix + "Confirm.label",
                                                   [brandShortName]);
    let notNowButton = shellBundle.getString(buttonPrefix + "NotNow.label");

    if (useNotificationBar) {
      let promptMessage = shellBundle.getFormattedString("setDefaultBrowserMessage2",
                                                         [brandShortName]);
      let optionsMessage = shellBundle.getString("setDefaultBrowserOptions.label");
      let optionsKey = shellBundle.getString("setDefaultBrowserOptions.accesskey");

      let neverLabel = shellBundle.getString("setDefaultBrowserNever.label");
      let neverKey = shellBundle.getString("setDefaultBrowserNever.accesskey");

      let yesButtonKey = shellBundle.getString("setDefaultBrowserConfirm.accesskey");
      let notNowButtonKey = shellBundle.getString("setDefaultBrowserNotNow.accesskey");

      let notificationBox = win.document.getElementById("high-priority-global-notificationbox");

      this._createPopup(win, {
        label: notNowButton,
        accesskey: notNowButtonKey
      }, {
        label: neverLabel,
        accesskey: neverKey
      });

      let buttons = [
        {
          label: yesButton,
          accessKey: yesButtonKey,
          callback: () => {
            this.setAsDefault();
            this.closePrompt();
          }
        },
        {
          label: optionsMessage,
          accessKey: optionsKey,
          popup: this.OPTIONPOPUP
        }
      ];

      let iconPixels = win.devicePixelRatio > 1 ? "32" : "16";
      let iconURL = "chrome://branding/content/icon" + iconPixels + ".png";
      const priority = notificationBox.PRIORITY_WARNING_HIGH;
      let callback = this._onNotificationEvent.bind(this);
      this._notification = notificationBox.appendNotification(promptMessage, "default-browser",
                                                              iconURL, priority, buttons,
                                                              callback);
    } else {
      // Modal prompt
      let promptTitle = shellBundle.getString("setDefaultBrowserTitle");
      let promptMessage = shellBundle.getFormattedString("setDefaultBrowserMessage",
                                                         [brandShortName]);
      let askLabel = shellBundle.getFormattedString("setDefaultBrowserDontAsk",
                                                    [brandShortName]);

      let ps = Services.prompt;
      let shouldAsk = { value: true };
      let buttonFlags = (ps.BUTTON_TITLE_IS_STRING * ps.BUTTON_POS_0) +
                        (ps.BUTTON_TITLE_IS_STRING * ps.BUTTON_POS_1) +
                        ps.BUTTON_POS_0_DEFAULT;
      let rv = ps.confirmEx(win, promptTitle, promptMessage, buttonFlags,
                            yesButton, notNowButton, null, askLabel, shouldAsk);
      if (rv == 0) {
        this.setAsDefault();
      } else if (!shouldAsk.value) {
        if (ShellService) {
          ShellService.shouldCheckDefaultBrowser = false;
        }
      }
    }
  },

  _onNotificationEvent: function(eventType) {
    if (eventType == "removed") {
      let doc = this._notification.ownerDocument;
      let popup = doc.getElementById(this.OPTIONPOPUP);
      popup.removeEventListener("command", this);
      popup.remove();
      delete this._notification;
    }
  },
};

function delayedStartup() {
  Services.obs.removeObserver(delayedStartup, "browser-delayed-startup-finished");
  if (!ShellService)
    return;

  if (!Services.prefs.getBoolPref("extensions.partnerdefaults.firstRunDone")) {
    // first run only, display the bookmarks toolbar
    let win = RecentWindow.getMostRecentBrowserWindow();

    let showBookmarksToolbar = Services.prefs.getBoolPref("extensions.partnerdefaults.showBookmarksToolbar");
    if (showBookmarksToolbar) {
      let bookmarksToolbar = win.document.getElementById("PersonalToolbar");
      win.setToolbarVisibility(bookmarksToolbar, true);
      // don't force show again
      Services.prefs.getBoolPref("extensions.partnerdefaults.showBookmarksToolbar", false);
    }
  }

  // if this is true we let the browser do its thing, ignore our modifications
  if (Services.prefs.getBoolPref("browser.shell.checkDefaultBrowser"))
    return;

  let firstRunDate;
  let _prefs = new Preferences("extensions.partnerdefaults.");
  try {
    let checkDefaultDays = Services.prefs.getIntPref("extensions.partnerdefaults.checkDefaultDays");
    // if the pref was set, firstRun has happened
    firstRunDate = CommonUtils.getDatePref(_prefs, "firstRunDate", -1, console.log,
                                           OLDEST_ALLOWED_YEAR);
    // sigh.  oldest allowed year doesnt help us here
    if (firstRunDate.getTime() < 0)
      throw("no first run date");

    if (checkDefaultDays > 0) {
      let now = new Date();
      if (firstRunDate.getTime() + checkDefaultDays * MILLISECONDS_PER_DAY <= now.getTime() ) {
        // the next startup will prompt for default
        Services.prefs.setBoolPref("browser.shell.checkDefaultBrowser", true);
      }
      return;
    }
  } catch(e) {
    // this is firstRun
    CommonUtils.setDatePref(_prefs, "firstRunDate", new Date(), OLDEST_ALLOWED_YEAR);
  }

  // in minutes.  if zero, no prompt in first session
  let delay = Services.prefs.getIntPref("extensions.partnerdefaults.checkDefaultDelay") * 60 * 1000;
  if (!delay)
    return;

  // XXX Leak the timer else it may not fire
  timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
  timer.initWithCallback(function() {
    timer = null;
    // just in case the user set default before the timer
    let isDefault = ShellService.isDefaultBrowser(true, false);
    if (isDefault)
      return;
    Services.tm.mainThread.dispatch(function() {
      DefaultBrowserCheck.prompt(RecentWindow.getMostRecentBrowserWindow());
    }.bind(this), Ci.nsIThread.DISPATCH_NORMAL);
  }, delay, Ci.nsITimer.TYPE_ONE_SHOT);
}

function startup(data, reason) {
  Services.obs.addObserver(delayedStartup, "browser-delayed-startup-finished", false);
}

function shutdown() {
  // if user shutsdown before delayed default check, next startup will will
  // operate normally without delayed check
  if (!ShellService)
    return;

  // if this is true we let the browser do its thing, ignore our modifications
  if (Services.prefs.getBoolPref("browser.shell.checkDefaultBrowser"))
    return;
  // we only check if we should modify checkDefaultBrowser from here on the very first run
  if (Services.prefs.getBoolPref("extensions.partnerdefaults.firstRunDone"))
    return;

  let isDefault = ShellService.isDefaultBrowser(true, false);
  if (isDefault)
    return;

  // if we're using a delayed default check for first run, reset checkDefaultBrowser
  let checkDefaultDays = Services.prefs.getIntPref("extensions.partnerdefaults.checkDefaultDays");
  if (checkDefaultDays == 0) {
    Services.prefs.setBoolPref("browser.shell.checkDefaultBrowser", true);
  }
  Services.prefs.setBoolPref("extensions.partnerdefaults.firstRunDone", true);
}

function install() {
}

function uninstall() {
}
